# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: keytable.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='keytable.proto',
  package='pskManifest',
  syntax='proto3',
  serialized_pb=_b('\n\x0ekeytable.proto\x12\x0bpskManifest\"*\n\x0b\x44\x65viceEntry\x12\x0b\n\x03urn\x18\x01 \x01(\t\x12\x0e\n\x06opaque\x18\x02 \x01(\x0c\"5\n\x08KeyTable\x12)\n\x07\x65ntries\x18\x01 \x03(\x0b\x32\x18.pskManifest.DeviceEntryb\x06proto3')
)




_DEVICEENTRY = _descriptor.Descriptor(
  name='DeviceEntry',
  full_name='pskManifest.DeviceEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='urn', full_name='pskManifest.DeviceEntry.urn', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='opaque', full_name='pskManifest.DeviceEntry.opaque', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=31,
  serialized_end=73,
)


_KEYTABLE = _descriptor.Descriptor(
  name='KeyTable',
  full_name='pskManifest.KeyTable',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='entries', full_name='pskManifest.KeyTable.entries', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=75,
  serialized_end=128,
)

_KEYTABLE.fields_by_name['entries'].message_type = _DEVICEENTRY
DESCRIPTOR.message_types_by_name['DeviceEntry'] = _DEVICEENTRY
DESCRIPTOR.message_types_by_name['KeyTable'] = _KEYTABLE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

DeviceEntry = _reflection.GeneratedProtocolMessageType('DeviceEntry', (_message.Message,), dict(
  DESCRIPTOR = _DEVICEENTRY,
  __module__ = 'keytable_pb2'
  # @@protoc_insertion_point(class_scope:pskManifest.DeviceEntry)
  ))
_sym_db.RegisterMessage(DeviceEntry)

KeyTable = _reflection.GeneratedProtocolMessageType('KeyTable', (_message.Message,), dict(
  DESCRIPTOR = _KEYTABLE,
  __module__ = 'keytable_pb2'
  # @@protoc_insertion_point(class_scope:pskManifest.KeyTable)
  ))
_sym_db.RegisterMessage(KeyTable)


# @@protoc_insertion_point(module_scope)
